@@ -136,12 +136,12 @@ ORBit_handle_exception_array (GIOPRecvBuffer     *rb,
 
 		dprintf (MESSAGES, "user exception\n");
 
-		for (i = 0; i < types->_length; i++) {
+		for (i = 0; my_repoid && i < types->_length; i++) {
 			if (!strcmp (types->_buffer[i]->repo_id, my_repoid))
 				break;
 		}
 
-		if (!types || types->_length == 0) {
+		if (!types || types->_length == 0 || i >= types->_length) {
 			/* weirdness; they raised an exception that we don't
 			   know about */
 			CORBA_exception_set_system (
@@ -591,6 +591,9 @@ ORBit_small_invoke_stub (CORBA_Object       obj,
 	GIOPRecvBuffer         *recv_buffer = NULL;
 	CORBA_Object            xt_proxy = CORBA_OBJECT_NIL;
 	ORBitPolicy            *invoke_policy = CORBA_OBJECT_NIL;
+	gboolean                timeout = FALSE;
+
+	CORBA_exception_init (ev);
 
 	if (!obj) {
 		dprintf (MESSAGES, "Cannot invoke method on null object\n");
@@ -599,8 +602,6 @@ ORBit_small_invoke_stub (CORBA_Object       obj,
 		goto clean_out;
 	}
 
-	CORBA_exception_init (ev);
-
 	if ((invoke_policy = ORBit_object_get_policy (obj)))
 		ORBit_policy_push (invoke_policy);
 
@@ -654,7 +655,9 @@ ORBit_small_invoke_stub (CORBA_Object       obj,
 		goto clean_out;
 	}
 
-	recv_buffer = giop_recv_buffer_get (&mqe);
+	recv_buffer = giop_recv_buffer_get (&mqe, &timeout);
+	if (timeout)
+		goto timeout_exception;
 
 	switch (orbit_small_demarshal (obj, &cnx, recv_buffer, ev,
 				       ret, m_data, args))
@@ -699,6 +702,12 @@ ORBit_small_invoke_stub (CORBA_Object       obj,
 	CORBA_exception_set_system (ev, ex_CORBA_COMM_FAILURE,
 				    completion_status);
 	goto clean_out;
+
+ timeout_exception:
+	tprintf ("[System exception timeout] )");
+	CORBA_exception_set_system (ev, ex_CORBA_TIMEOUT,
+				    CORBA_COMPLETED_NO);
+	goto clean_out;
 }
 
 void
@@ -1140,6 +1149,11 @@ async_recv_cb (ORBitAsyncQueueEntry *aqe)
 		CORBA_exception_set_system (ev, ex_CORBA_COMM_FAILURE,
 					    aqe->completion_status);
 
+	if (aqe->mqe.cnx &&
+	    aqe->mqe.cnx->parent.status == LINK_TIMEOUT)
+		CORBA_exception_set_system (ev, ex_CORBA_TIMEOUT,
+					    aqe->completion_status);
+
 	if (aqe->fn)
 		aqe->fn (aqe->obj, aqe->m_data, aqe, aqe->user_data, ev);
 
@@ -1339,7 +1353,7 @@ ORBit_small_unlisten_for_broken_full (CORBA_Object obj,
 	else {
 		GIOPConnection *cnx;
 
-		cnx = ORBit_object_get_connection (obj);
+		cnx = ORBit_object_peek_connection (obj);
 
 		if (cnx) {
 			ret = get_status (cnx);
@@ -1384,7 +1398,7 @@ void
 ORBit_small_connection_unref (ORBitConnection *cnx)
 {
 	if (cnx)
-		giop_connection_unref (cnx);
+		giop_connection_unref (GIOP_CONNECTION (cnx));
 }
 
 void
