@@ -52,7 +52,7 @@ PingPongServer_opOneWay (PortableServer_Servant servant,
 			 CORBA_Environment     *ev)
 {
 	/* Do nothing, but try and confuse the queue */
-	linc_main_iteration (FALSE);
+	link_main_iteration (FALSE);
 }
 
 static void
@@ -144,6 +144,7 @@ POA_test_PingPongServer__vepv PingPongServer_vepv = {
 static POA_test_PingPongServer *
 create_ping_pong_servant (void)
 {
+	CORBA_Environment ev[1];
 	test_PingPongServer_Servant *servant;
 	
 	servant = g_new0 (test_PingPongServer_Servant, 1);
@@ -151,5 +152,10 @@ create_ping_pong_servant (void)
 
 	servant->registered = CORBA_OBJECT_NIL;
 
+	CORBA_exception_init (ev);
+	POA_test_PingPongServer__init (servant, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_exception_free (ev);
+
 	return (POA_test_PingPongServer *) servant;
 };
